#----------------------------------------------------------------------------
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/system/Lock.pm,v 1.12 2008/10/06 05:09:33 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2008/10/06 05:09:33 $
# $Revision: 1.12 $
#
#
# Provides: package Lock
#
# Functionality: Simple file based locking system
#
# Authors: 011787
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
# Important notes about usage - see (A) to (C) below.
#
# This code is written as an advisory locking mechanism
# based simply on a caller providing a "lockname"
#
#(A) It is not intended to allow a lock to be held across perl script
# invocation boundaries.
# (ie dont take a lock out in a perl script and then run the script
# again and expect to be able to release the lock.)
#
#(B) Locks do work between script instances running concurently
# (ie contention for a scare resource, if script 'x' is running and needs
# to access a tape drive, requests a lock called "tapedrive1", script 'y',
# or another instance of 'x', can request the same lock "tapedrive1" and
# will be made to wait.)
#
#(C) To remove/reduce the instance of 'stuck locks' the locking
# mechanism will take locks by force if the GetLock request takes longer
# than VCSDEF_LOCKS_SLEEP_PERIOD * VCSDEF_LOCKS_SLEEP_ITERATIONS seconds
#-----------------------------------------------------------------------------

#------------------
# version calls.
#------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/system/Lock.pm,v 1.12 2008/10/06 05:09:33 aatre Exp $::ETVHeader");
#------------------------
# end of version calls.
#------------------------

#------------------------------
# Declare the package name.
#------------------------------
package Lock;

#------------------------------------
# Include all the packages required.
#------------------------------------
use LogInt;

#----------------------------------------
# Turn on strcit on vars, refs and subs. 
#----------------------------------------
use strict;

#------------------------------------------------------
# locks hash.
# store the LockName against a LockId (random number)
#------------------------------------------------------
%::ghLockHash = ();

#-----------------------------------------------------------------
# const defs.
# 1) directory in which to put the lock files.
# 2) period to wait for lock aquire attempts
# 3) max times to attempt to aquire lock (then forcibly take it)
# 4) large number for use in generating 'unique' lock Ids
#-----------------------------------------------------------------
use constant VCSDEF_LOCKS_DIR              => "/tmp/.Lock";
use constant VCSDEF_LOCKS_SLEEP_PERIOD     => 3;
use constant VCSDEF_LOCKS_SLEEP_ITERATIONS => 5;
use constant VCSDEF_MAX_LOCK_ID            => 10000000;


=pod

----------------------------------------------------------------------------
Function: Boolean GetLock( IN String LockName )

Description:
  File based locking mechanism to acquire a lock.

Parameters:
  LockName : Name of the lock file to acquire.

Return Value:
  Boolean value in the set of {0, 1) where
          0 - false
          1 - true
----------------------------------------------------------------------------

=cut

sub GetLock {  

  my ( $sLockName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10244,
                   "Lock",
                   "GetLock",
                   "Subroutine %s is called with: LockName [%s]",
                   $sMyName, $sLockName );

  LogInt::Message( LogInt::LLTrace,
                   10245,
                   "Lock",
                   "GetLock",
                   "Started GetLock for lockname:[$sLockName]" );

  #-------------------------------------------
  # make the locks dir if it doesnt exist. 
  #-------------------------------------------
  if(-d VCSDEF_LOCKS_DIR) {
    #ok continue
  } 
  else {
    #----------------------------------------
    # first time run - make the locks dir.
    #----------------------------------------
    if(mkdir(VCSDEF_LOCKS_DIR, 0777)) {
      chmod 0777, VCSDEF_LOCKS_DIR; #chmod to 777 
      LogInt::Message( LogInt::LLTrace,
                       10246,
                       "Lock",
                       "GetLock",
                       "Created locks dir [%s]", VCSDEF_LOCKS_DIR );
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10247,
                       "Lock",
                       "GetLock",
                       "Failed to create locks dir [%s]", VCSDEF_LOCKS_DIR  );
      return 0;
    }
  }

  #-----------------------------------------
  # make sure we didn't already aquire it.
  #-----------------------------------------
  my $iLockIdTest = $::ghLockHash{$sLockName};

  if($iLockIdTest) {
    LogInt::Message( LogInt::LLError, 
                     10248,
                     "Lock",
                     "GetLock",
                     "Failing attempt to re-aquire lock [$sLockName], duplicate request call?" );
    return 0;
  }

  #---------------------------------------------------------------------------------
  # Attempt to aquire the lock, if we can't then start to wait.
  # if lock persists for VCSDEF_LOCKS_SLEEP_PERIOD * VCSDEF_LOCKS_SLEEP_ITERATIONS 
  # seconds then this routine will forcibly aquire the lock and continue
  #---------------------------------------------------------------------------------
  my $sLockFile = VCSDEF_LOCKS_DIR . "/$sLockName";
  my $bWaitingForLock = undef;
  if(-e $sLockFile) {
    LogInt::Message( LogInt::LLTrace,
                     10249,
                     "Lock",
                     "GetLock",
                     "waiting for lockname:[$sLockName]" );
    $bWaitingForLock = 1;
  } 
  else {
    $bWaitingForLock = 0;
  }

  my $iCount = 0;
  while($bWaitingForLock and $iCount < VCSDEF_LOCKS_SLEEP_ITERATIONS) {
    sleep VCSDEF_LOCKS_SLEEP_PERIOD;
    if(-e $sLockFile) {
      LogInt::Message( LogInt::LLTraceTrace,
                       10250,
                       "Lock",
                       "GetLock",
                       "still waiting for lockname:[$sLockName]" );
    } 
    else {
      $bWaitingForLock=0;
    }
    $iCount++;
  }

  #-------------------------
  # did we get the lock.
  #-------------------------
  if($bWaitingForLock) {
    if(unlink($sLockFile)) {
      LogInt::Message( LogInt::LLWarn,
                       10251,
                       "Lock",
                       "GetLock",
                       "Taking lock [$sLockName] by force" );
    } 
    else {
      LogInt::Message( LogInt::LLError, 
                       10252,
                       "Lock",
                       "GetLock",
                       "Failed to take lock [$sLockName] by force; could not unlink lock file [$sLockFile]" );
			   
      return 0;
    }
  }

  if(open(LOCK_FILE, ">$sLockFile")) {
    #-------------------------------------------
    # generate a LockId and write it to file.
    #-------------------------------------------
    my $iLockId = int rand VCSDEF_MAX_LOCK_ID;
    print LOCK_FILE $iLockId;
    close(LOCK_FILE);

    #-------------------------
    # add LockId to hash.
    #-------------------------
    $::ghLockHash{$sLockName} = $iLockId;

    LogInt::Message( LogInt::LLTrace,
                     10253,
                     "Lock",
                     "GetLock",
                     "successfully aquired lockname:[$sLockName], lockid:[$iLockId], lockfile:[$sLockFile]" );

    return 1; #success
  } 
  else {
    LogInt::Message( LogInt::LLError, 
                     10254,
                     "Lock",
                     "GetLock",
                     "Failed to open like file [$sLockFile] for writing" );
    return 0;
  }
  
}


=pod

----------------------------------------------------------------------------
Function: Boolean ReleaseLock( IN String LockName )

Description:
  File based locking mechanism to release a lock.

Parameters:
  LockName : Name of the lock to be released.

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub ReleaseLock {

  my ( $sLockName ) = @_;
  
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10255,
                   "Lock",
                   "ReleaseLock",
                   "Subroutine %s is called with: LockName [%s]",
                   $sMyName, $sLockName );

  LogInt::Message( LogInt::LLTrace, 
                   10256,
                   "Lock",
                   "ReleaseLock",
                   "Started ReleaseLock for lockname:[$sLockName]" );

  #-------------------------------------------------------------
  # check the hash to make sure we were the ones who locked it.
  #-------------------------------------------------------------
  my $iLockIdTest = $::ghLockHash{$sLockName};

  if($iLockIdTest) {
    #-------------------------
    # delete our hash key.
    #-------------------------
    delete $::ghLockHash{$sLockName};
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10257,
                     "Lock",
                     "ReleaseLock",
                     "Failing attempt to release lockname:[$sLockName], duplicate release call?" );
    return 0;
  }

  my $sLockFile = VCSDEF_LOCKS_DIR . "/$sLockName";

  #------------------------------------------------
  # check if we are releasing a lock we aquired.
  #------------------------------------------------
  if(open(LOCK_FILE, "<$sLockFile")) {
    my $iLockIdTestRead = <LOCK_FILE>;
    close(LOCK_FILE);
    chomp $iLockIdTestRead;
    if($iLockIdTestRead == $iLockIdTest) {
      if(unlink($sLockFile)) {
        LogInt::Message( LogInt::LLTrace,
	                 10258,
                         "Lock",
                         "ReleaseLock",
                         "Released our lock, lockname:[$sLockName]" );
        return 1; #success
      } 
      else {
        LogInt::Message( LogInt::LLError,
	                 10259,
                         "Lock",
                         "ReleaseLock",
                         "Failed to release lock [$sLockName] by force; could not unlink lock file [$sLockFile]" );
			     
        return 0;
      }
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10260,
                       "Lock",
                       "ReleaseLock",
                       "Failed to release a lock lockname:[$sLockName] with lockid:[$iLockIdTestRead], we did not aquire it" );
			   
      return 0;
    }
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10261,
                     "Lock",
                     "ReleaseLock",
                     "Failed to read a lock file lockname:[$sLockName]" );
    return 0;
  }
}


1;

